#include "dfontinfoscrollarea.h"
#include <QDebug>
#include <DFontSizeManager>
#include <DApplication>
#include <DApplicationHelper>
#include <QTimer>

#define LAEBL_TEXT_WIDTH   165
#define TITLE_VISIBLE_WIDTH 90
#define INFO_VISIBLE_WIDTH 180
#define IS_NEED_ELLIPSIS 30 //是否需要省略号

dfontinfoscrollarea::dfontinfoscrollarea(DFontPreviewItemData *pData,  DWidget *parent)
    : DFrame(parent)
    , m_fontInfo(pData)
{
    installEventFilter(this);

    setFrameShape(QFrame::NoFrame);
    initUi();
}

void dfontinfoscrollarea::initUi()
{
    auto gridLayout = new QGridLayout;
    gridLayout->setContentsMargins(0, 6, 0, 6);
    gridLayout->setSpacing(3);

    QLocale locale;
    QFontMetrics fm(font());

    if (locale.language() == QLocale::English) {
        m_leftminwidth = fm.horizontalAdvance(("Time modified"));
    } else if (locale.language() == QLocale::Chinese) {
        m_leftminwidth = fm.horizontalAdvance("页面大小");
    }

    createLabel(gridLayout, 0, DApplication::translate("FontDetailDailog", "Style")
                , m_fontInfo->fontInfo.styleName);
    createLabel(gridLayout, 1, DApplication::translate("FontDetailDailog", "Type")
                , DApplication::translate("FontDetailDailog", m_fontInfo->fontInfo.type.toLatin1()));
    if (m_fontInfo->fontInfo.version.isEmpty()) {
        QString version = "Copyright 2014~2015 Adobe Syste-ms Incorporated (http://www.adob.com/), with Reserved Font Name cc Source.";
        createLabel(gridLayout, 2, DApplication::translate("FontDetailDailog", "Version"), version);
    } else {
        createLabel(gridLayout, 2, DApplication::translate("FontDetailDailog", "Version")
                    , m_fontInfo->fontInfo.version);
    }
    if (m_fontInfo->fontInfo.description.isEmpty()) {
        createLabel(gridLayout, 3, DApplication::translate("FontDetailDailog", "Description")
                    , DApplication::translate("FontDetailDailog", "Unknown"));
    } else {
        createLabel(gridLayout, 3, DApplication::translate("FontDetailDailog", "Description")
                    , m_fontInfo->fontInfo.description);
    }
    createLabel(gridLayout, 4, DApplication::translate("FontDetailDailog", "Full name"), m_fontInfo->fontInfo.fullname);
    createLabel(gridLayout, 5, DApplication::translate("FontDetailDailog", "Ps name"), m_fontInfo->fontInfo.psname);
    createLabel(gridLayout, 6, DApplication::translate("FontDetailDailog", "Trademark"), m_fontInfo->fontInfo.trademark);

    auto vLayout = new QVBoxLayout;
    vLayout->setContentsMargins(10, 10, 10, 10);
    basicLabel = new DLabel(DApplication::translate("FontDetailDailog", "Basic info"));
    DFontSizeManager::instance()->bind(basicLabel, DFontSizeManager::T6);

    vLayout->addWidget(basicLabel);
    vLayout->addItem(gridLayout);
    vLayout->addStretch(1);
    this->setLayout(vLayout);
}

bool dfontinfoscrollarea::eventFilter(QObject *obj, QEvent *e)
{
    if (e->type() == QEvent::FontChange) {
        updateText();
        QTimer::singleShot(0, [ = ] {
            autoHeight();
        });
    }
    return  DFrame::eventFilter(obj, e);
}

//初始化信息页面label/*539*/
void dfontinfoscrollarea::createLabel(QGridLayout *layout, const int &index, const QString &objName, const QString &sData)
{
    QString str = objName;
    DLabel *label = new DLabel(elideText(str), this);
    DFontSizeManager::instance()->bind(label, DFontSizeManager::T8);
    label->setAlignment(Qt::AlignTop);

    label->setFixedWidth(100);
    layout->addWidget(label, index, 0);
    if (pTitleMap.find(label) == pTitleMap.end()) {
        pTitleMap.insert(std::pair<QLabel *, QString>(label, objName));
    }
    if (sData == "") {
        DLabel *labelText = new DLabel(this);
        DFontSizeManager::instance()->bind(labelText, DFontSizeManager::T8);
        labelText->setText(DApplication::translate("FontDetailDailog", "Unknown"));
        labelText->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
        layout->addWidget(labelText, index, 1);
    } else {
        DFrame *widgets = addTitleFrame(sData, objName);
        widgets->setFrameShape(QFrame::NoFrame);
        layout->addWidget(widgets, index, 1);
    }
}

//增加内容信息frame/*539*/
DFrame *dfontinfoscrollarea::addTitleFrame(const QString &sData, const QString &objName)
{
    Q_UNUSED(objName);
    DFrame *m_textShowFrame = new DFrame(this);
    QString ts = elideText(sData, this->font(), INFO_VISIBLE_WIDTH);

    QVBoxLayout *vLayout = new QVBoxLayout;

    QLabel *label = new QLabel(ts, m_textShowFrame);
    label->setFixedWidth(INFO_VISIBLE_WIDTH);
    DFontSizeManager::instance()->bind(label, DFontSizeManager::T8);
    if (pLabelMap.find(label) == pLabelMap.end()) {
        pLabelMap.insert(std::pair<QLabel *, QString>(label, sData));
    }

    label->setAlignment(Qt::AlignLeft);
    label->adjustSize();

    vLayout->addWidget(label);

    vLayout->addStretch(1);
    vLayout->setContentsMargins(0, 0, 0, 0);
    QHBoxLayout *textShowLayout = new QHBoxLayout;
    textShowLayout->setContentsMargins(0, 0, 0, 0);
    textShowLayout->setSpacing(0);
    textShowLayout->addLayout(vLayout);
    m_textShowFrame->setLayout(textShowLayout);
    vLayout->setContentsMargins(0, 0, 0, 0);

    textShowLayout->addStretch(1);

    m_textShowFrame->setContentsMargins(0, 0, 0, 0);

    return m_textShowFrame;
}

//重绘信息页面/*539(勿删勿动!)*/
void dfontinfoscrollarea::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);
}

//用于信息页面title名称长度判断/*539*/
QString dfontinfoscrollarea::elideText(QString &titleName) const
{
    QFont font = this->font();
    QFontMetrics fontMetric(font);
    QString finalTitle = "";
    QString m_curTitle = "";
    int curWidth = 0;

    for (auto str : titleName) {
        if (str == "\t") {
            curWidth  += fontMetric.width("a");
        } else {
            curWidth  += fontMetric.width(str);
        }
        m_curTitle += str;
        if (curWidth > TITLE_VISIBLE_WIDTH) {
            if (m_curTitle == titleName) {
                finalTitle = titleName;
                break;
            } else {
                if (fontMetric.width("...") > IS_NEED_ELLIPSIS) {
                    finalTitle = m_curTitle;
                } else {
                    finalTitle =   m_curTitle.append("...");
                }
                break;
            }
        } else {
            finalTitle = titleName;
        }
    }
    return finalTitle;
}

//用于信息页面详细信息部分换行判断/*539*/
QString dfontinfoscrollarea::elideText(const QString &text, const QFont &font, int nLabelSize)
{
    QFontMetrics fm(font);
    QString strText = text;
    int n_TextSize = fm.width(strText);
    int count  = 0;
    if (n_TextSize > nLabelSize) {
        int n_position = 0;
        long n_curSumWidth = 0;
        for (int i = 0; i < strText.size(); i++) {
            n_curSumWidth += fm.width(strText.at(i));
            if (n_curSumWidth > nLabelSize * (count + 1)) {
                n_position = i;
                strText.insert(n_position, "\n");
                count++;
            }
        }
    }
    return strText;
}

//重绘信息页面/*539(勿删勿动!)*/
void dfontinfoscrollarea::updateText()
{
    for (auto pTltle : pTitleMap) {
        if (!pTltle.first) {
            continue;
        }
        QString text = pTltle.second;
        QString newtext = elideText(text);
        pTltle.first->setText(newtext);
    }
    for (auto plabeliter : pLabelMap) {
        if (!plabeliter.first) {
            continue;
        }
        QString text = plabeliter.second;
        QString newtext = elideText(text, this->font(), INFO_VISIBLE_WIDTH);
        plabeliter.first->setText(newtext);
    }
}

void dfontinfoscrollarea::autoHeight()
{
    int m_totalHeight = 0;
    for (auto plabeliter : pLabelMap) {
        if (!plabeliter.first) {
            continue;
        }
        m_totalHeight = m_totalHeight + plabeliter.first->height();
    }
    emit m_signalManager->sizeChange(m_totalHeight + 76 + basicLabel->height());
}
